@@ -18,9 +18,10 @@ import json
 import os
 import six
 import tempfile
+import yaml
 
+from keystoneclient import exceptions as kscexc
 import mock
-
 from openstackclient.common import exceptions as oscexc
 
 from tripleoclient import constants
@@ -104,6 +105,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         arglist = ['--templates', '--ceph-storage-scale', '3']
         verifylist = [
             ('templates', '/usr/share/openstack-tripleo-heat-templates/'),
+            ('ceph_storage_scale', 3)
         ]
 
         mock_create_cephx_key.return_value = "cephx_key"
@@ -129,56 +131,47 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             "id": "network id"
         }
         mock_create_env.return_value = "/fake/path"
-        mock_process_multiple_env.return_value = [{}, "env"]
+        mock_env = fakes.create_env()
+        mock_process_multiple_env.return_value = [{}, mock_env]
         mock_get_templte_contents.return_value = [{}, "template"]
         wait_for_stack_ready_mock.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        baremetal = clients.tripleoclient.baremetal
+        baremetal = clients.baremetal
         baremetal.node.list.return_value = range(10)
 
         expected_parameters = {
             'AdminPassword': 'password',
             'AdminToken': 'password',
-            'BlockStorageImage': 'overcloud-full',
+            'AodhPassword': 'password',
             'CeilometerMeteringSecret': 'password',
             'CeilometerPassword': 'password',
             'CephStorageCount': 3,
-            'CephStorageImage': 'overcloud-full',
-            'CinderISCSIHelper': 'lioadm',
             'CinderPassword': 'password',
-            'CloudName': 'overcloud',
-            'controllerImage': 'overcloud-full',
             'ExtraConfig': '{}',
             'GlancePassword': 'password',
+            'HAProxyStatsPassword': 'password',
             'HeatPassword': 'password',
             'HeatStackDomainAdminPassword': 'password',
             'HypervisorNeutronPhysicalBridge': 'br-ex',
             'HypervisorNeutronPublicInterface': 'nic1',
             'NeutronAllowL3AgentFailover': False,
-            'NeutronBridgeMappings': 'datacentre:br-ex',
             'NeutronDhcpAgentsPerNetwork': 1,
             'NeutronDnsmasqOptions': 'dhcp-option-force=26,1400',
             'NeutronFlatNetworks': 'datacentre',
             'NeutronL3HA': False,
             'NeutronMetadataProxySharedSecret': 'password',
-            'NeutronNetworkVLANRanges': 'datacentre:1:1000',
             'NeutronPassword': 'password',
             'NeutronPublicInterface': 'nic1',
-            'NovaImage': 'overcloud-full',
             'NovaPassword': 'password',
             'NtpServer': '',
-            'OvercloudBlockStorageFlavor': 'baremetal',
-            'OvercloudCephStorageFlavor': 'baremetal',
-            'OvercloudComputeFlavor': 'baremetal',
-            'OvercloudControlFlavor': 'baremetal',
-            'OvercloudSwiftStorageFlavor': 'baremetal',
+            'RabbitPassword': 'password',
+            'RedisPassword': 'password',
             'SaharaPassword': 'password',
             'SnmpdReadonlyUserPassword': 'PASSWORD',
             'SwiftHashSuffix': 'password',
             'SwiftPassword': 'password',
-            'SwiftStorageImage': 'overcloud-full',
             'DeployIdentifier': 123456789,
         }
 
@@ -201,8 +194,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         self.assertEqual(kwargs['files'], {})
         self.assertEqual(kwargs['template'], 'template')
-        self.assertEqual(kwargs['environment'], 'env')
+        self.assertEqual(kwargs['environment'], mock_env)
         self.assertEqual(kwargs['stack_name'], 'overcloud')
+        self.assertEqual(kwargs['clear_parameters'],
+                         mock_env['parameter_defaults'].keys())
 
         mock_get_templte_contents.assert_called_with(
             '/usr/share/openstack-tripleo-heat-templates/' +
@@ -217,12 +212,14 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                 '_validate_args')
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_create_parameters_env')
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_deploy_postconfig')
     @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
                 autospec=True)
     @mock.patch('tripleoclient.utils.generate_overcloud_passwords')
     @mock.patch('tripleoclient.utils.create_overcloudrc')
+    @mock.patch('os_cloud_config.utils.clients.get_nova_bm_client',
+                autospec=True)
+    @mock.patch('os_cloud_config.utils.clients.get_keystone_client',
+                autospec=True)
     @mock.patch('os_cloud_config.keystone.setup_endpoints', autospec=True)
     @mock.patch('time.sleep', return_value=None)
     @mock.patch('os_cloud_config.keystone.initialize', autospec=True)
@@ -251,16 +248,17 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                         wait_for_stack_ready_mock,
                         mock_remove_known_hosts, mock_keystone_initialize,
                         mock_sleep, mock_setup_endpoints,
+                        mock_get_keystone_client, mock_get_nova_bm_client,
                         mock_create_overcloudrc,
                         mock_generate_overcloud_passwords,
                         mock_create_tempest_deployer_input,
-                        mock_deploy_postconfig,
                         mock_create_parameters_env, mock_validate_args,
                         mock_breakpoints_cleanup):
 
         arglist = ['--templates', '--ceph-storage-scale', '3']
         verifylist = [
             ('templates', '/usr/share/openstack-tripleo-heat-templates/'),
+            ('ceph_storage_scale', 3)
         ]
 
         mock_create_cephx_key.return_value = "cephx_key"
@@ -289,65 +287,53 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             "id": "network id"
         }
         mock_create_env.return_value = "/fake/path"
-        mock_process_multiple_env.return_value = [{}, "env"]
+        mock_env = fakes.create_env()
+        mock_process_multiple_env.return_value = [{}, mock_env]
         mock_get_templte_contents.return_value = [{}, "template"]
         wait_for_stack_ready_mock.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        baremetal = clients.tripleoclient.baremetal
+        baremetal = clients.baremetal
         baremetal.node.list.return_value = range(10)
 
         expected_parameters = {
             'AdminPassword': 'password',
             'AdminToken': 'password',
-            'BlockStorageImage': 'overcloud-full',
+            'AodhPassword': 'password',
             'CeilometerMeteringSecret': 'password',
             'CeilometerPassword': 'password',
             'CephAdminKey': 'cephx_key',
             'CephClusterFSID': 'uuid',
             'CephMonKey': 'cephx_key',
             'CephStorageCount': 3,
-            'CephStorageImage': 'overcloud-full',
-            'CinderISCSIHelper': 'lioadm',
             'CinderPassword': 'password',
-            'CloudName': 'overcloud',
-            'controllerImage': 'overcloud-full',
             'ExtraConfig': '{}',
             'GlancePassword': 'password',
+            'HAProxyStatsPassword': 'password',
             'HeatPassword': 'password',
             'HeatStackDomainAdminPassword': 'password',
             'HypervisorNeutronPhysicalBridge': 'br-ex',
             'HypervisorNeutronPublicInterface': 'nic1',
             'NeutronAllowL3AgentFailover': False,
-            'NeutronBridgeMappings': 'datacentre:br-ex',
             'NeutronDhcpAgentsPerNetwork': 1,
             'NeutronDnsmasqOptions': 'dhcp-option-force=26,1400',
             'NeutronEnableTunnelling': 'True',
             'NeutronFlatNetworks': 'datacentre',
             'NeutronL3HA': False,
             'NeutronNetworkType': 'gre',
-            'NeutronNetworkVLANRanges': 'datacentre:1:1000',
             'NeutronMetadataProxySharedSecret': 'password',
             'NeutronPassword': 'password',
             'NeutronPublicInterface': 'nic1',
-            'NeutronTunnelIdRanges': ['1:1000'],
             'NeutronTunnelTypes': 'gre',
-            'NeutronVniRanges': ['1:1000'],
-            'NovaComputeLibvirtType': 'kvm',
-            'NovaImage': 'overcloud-full',
             'NovaPassword': 'password',
             'NtpServer': '',
-            'OvercloudBlockStorageFlavor': 'baremetal',
-            'OvercloudCephStorageFlavor': 'baremetal',
-            'OvercloudComputeFlavor': 'baremetal',
-            'OvercloudControlFlavor': 'baremetal',
-            'OvercloudSwiftStorageFlavor': 'baremetal',
+            'RabbitPassword': 'password',
+            'RedisPassword': 'password',
             'SaharaPassword': 'password',
             'SnmpdReadonlyUserPassword': 'PASSWORD',
             'SwiftHashSuffix': 'password',
             'SwiftPassword': 'password',
-            'SwiftStorageImage': 'overcloud-full',
             'DeployIdentifier': 123456789,
         }
 
@@ -368,8 +354,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         self.assertEqual(kwargs['files'], {})
         self.assertEqual(kwargs['template'], 'template')
-        self.assertEqual(kwargs['environment'], 'env')
+        self.assertEqual(kwargs['environment'], mock_env)
         self.assertEqual(kwargs['stack_name'], 'overcloud')
+        self.assertEqual(kwargs['clear_parameters'],
+                         mock_env['parameter_defaults'].keys())
 
         mock_get_templte_contents.assert_called_with(
             '/usr/share/openstack-tripleo-heat-templates/' +
@@ -383,6 +371,11 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         mock_validate_args.assert_called_once_with(parsed_args)
 
+        mock_remove_known_hosts.assert_called_once_with('0.0.0.0')
+        mock_get_keystone_client.assert_called_once_with('admin', 'password',
+                                                         'admin',
+                                                         'http://0.0.0.0:8000')
+
     @mock.patch("heatclient.common.event_utils.get_events")
     @mock.patch('tripleo_common.update.add_breakpoints_cleanup_into_env')
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
@@ -434,7 +427,6 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         clients = self.app.client_manager
         orchestration_client = clients.tripleoclient.orchestration
         orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
-        orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
         mock_events.return_value = []
 
         mock_check_hypervisor_stats.return_value = {
@@ -447,13 +439,14 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             "id": "network id"
         }
         mock_create_env.return_value = "/fake/path"
-        mock_process_multiple_env.return_value = [{}, "env"]
+        mock_env = fakes.create_env()
+        mock_process_multiple_env.return_value = [{}, mock_env]
         mock_get_templte_contents.return_value = [{}, "template"]
         wait_for_stack_ready_mock.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        baremetal = clients.tripleoclient.baremetal
+        baremetal = clients.baremetal
         baremetal.node.list.return_value = range(10)
 
         with mock.patch('tempfile.mkstemp') as mkstemp:
@@ -469,8 +462,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         self.assertEqual(kwargs['files'], {})
         self.assertEqual(kwargs['template'], 'template')
-        self.assertEqual(kwargs['environment'], 'env')
+        self.assertEqual(kwargs['environment'], mock_env)
         self.assertEqual(kwargs['stack_name'], 'overcloud')
+        self.assertEqual(kwargs['clear_parameters'],
+                         mock_env['parameter_defaults'].keys())
 
         mock_get_templte_contents.assert_called_with(
             '/home/stack/tripleo-heat-templates/' +
@@ -629,6 +624,98 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         mock_create_tempest_deployer_input.assert_called_with()
 
+    @mock.patch("heatclient.common.event_utils.get_events")
+    @mock.patch('tripleo_common.update.add_breakpoints_cleanup_into_env')
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                '_deploy_postconfig')
+    @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
+                autospec=True)
+    @mock.patch('tripleoclient.utils.generate_overcloud_passwords')
+    @mock.patch('tripleoclient.utils.create_overcloudrc')
+    @mock.patch('os_cloud_config.keystone.setup_endpoints', autospec=True)
+    @mock.patch('time.sleep', return_value=None)
+    @mock.patch('os_cloud_config.keystone.initialize', autospec=True)
+    @mock.patch('tripleoclient.utils.remove_known_hosts', autospec=True)
+    @mock.patch('tripleoclient.utils.wait_for_stack_ready',
+                autospec=True)
+    @mock.patch('heatclient.common.template_utils.'
+                'process_multiple_environments_and_files', autospec=True)
+    @mock.patch('heatclient.common.template_utils.get_template_contents',
+                autospec=True)
+    @mock.patch('os_cloud_config.keystone_pki.generate_certs_into_json',
+                autospec=True)
+    @mock.patch('tripleoclient.utils.create_environment_file',
+                autospec=True)
+    @mock.patch('tripleoclient.utils.get_config_value', autospec=True)
+    @mock.patch('tripleoclient.utils.check_hypervisor_stats',
+                autospec=True)
+    def test_deploy_rhel_reg(self, mock_check_hypervisor_stats,
+                             mock_get_key,
+                             mock_create_env, generate_certs_mock,
+                             mock_get_templte_contents,
+                             mock_process_multiple_env,
+                             wait_for_stack_ready_mock,
+                             mock_remove_known_hosts,
+                             mock_keystone_initialize,
+                             mock_sleep, mock_setup_endpoints,
+                             mock_create_overcloudrc,
+                             mock_generate_overcloud_passwords,
+                             mock_create_tempest_deployer_input,
+                             mock_deploy_postconfig,
+                             mock_breakpoints_cleanup,
+                             mock_events):
+
+        arglist = ['--templates', '--rhel-reg',
+                   '--reg-sat-url', 'https://example.com',
+                   '--reg-method', 'satellite', '--reg-org', '123456789',
+                   '--reg-activation-key', 'super-awesome-key']
+        verifylist = [
+            ('templates', '/usr/share/openstack-tripleo-heat-templates/'),
+            ('rhel_reg', True),
+            ('reg_sat_url', 'https://example.com'),
+            ('reg_method', 'satellite'),
+            ('reg_org', '123456789'),
+            ('reg_activation_key', 'super-awesome-key')
+        ]
+
+        mock_generate_overcloud_passwords.return_value = self._get_passwords()
+        mock_process_multiple_env.return_value = [{}, fakes.create_env()]
+        mock_get_templte_contents.return_value = [{}, "template"]
+        wait_for_stack_ready_mock.return_value = True
+
+        clients = self.app.client_manager
+        orchestration_client = clients.tripleoclient.orchestration
+        orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
+        mock_events.return_value = []
+
+        mock_check_hypervisor_stats.return_value = {
+            'count': 4,
+            'memory_mb': 4096,
+            'vcpus': 8,
+        }
+        mock_get_key.return_value = "PASSWORD"
+        clients.network.api.find_attr.return_value = {
+            "id": "network id"
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        baremetal = clients.baremetal
+        baremetal.node.list.return_value = range(10)
+
+        result = self.cmd.take_action(parsed_args)
+        self.assertTrue(result)
+
+        args, kwargs = mock_process_multiple_env.call_args
+        self.assertIn(
+            '/usr/share/openstack-tripleo-heat-templates/extraconfig/pre_dep'
+            'loy/rhel-registration/rhel-registration-resource-registry.yaml',
+            args[0])
+        self.assertIn(
+            '/usr/share/openstack-tripleo-heat-templates/extraconfig/pre_dep'
+            'loy/rhel-registration/environment-rhel-registration.yaml',
+            args[0])
+
     def test_validate_args_correct(self):
         arglist = ['--templates',
                    '--neutron-network-type', 'nettype',
@@ -816,3 +903,97 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         self.assertFalse(mock_oc_endpoint.called)
         self.assertFalse(mock_create_ocrc.called)
         self.assertFalse(mock_create_tempest_deployer_input.called)
+
+    @mock.patch('tripleoclient.utils.check_nodes_count',
+                autospec=True)
+    @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
+                autospec=True)
+    @mock.patch('tripleoclient.utils.create_overcloudrc', autospec=True)
+    @mock.patch('tripleoclient.utils.get_overcloud_endpoint', autospec=True)
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                '_heat_deploy', autospec=True)
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                'set_overcloud_passwords', autospec=True)
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                '_pre_heat_deploy', autospec=True)
+    def test_answers_file(self,
+                          mock_pre_deploy,
+                          mock_set_overcloud_passwords,
+                          mock_heat_deploy,
+                          mock_oc_endpoint,
+                          mock_create_ocrc,
+                          mock_create_tempest_deployer_input,
+                          mock_check_nodes_count):
+        clients = self.app.client_manager
+        network_client = clients.network
+        network_client.stacks.get.return_value = None
+        net = network_client.api.find_attr('networks', 'ctlplane')
+        net.configure_mock(__getitem__=lambda x, y: 'testnet')
+
+        with tempfile.NamedTemporaryFile(mode="w+t") as answerfile:
+            yaml.dump(
+                {'templates': '/dev/null',
+                 'environments': ['/tmp/foo.yaml']
+                 },
+                answerfile
+            )
+            answerfile.flush()
+
+            arglist = ['--answers-file', answerfile.name,
+                       '--environment-file', '/tmp/environment.yaml',
+                       '--block-storage-scale', '3']
+            verifylist = [
+                ('answers_file', answerfile.name),
+                ('environment_files', ['/tmp/environment.yaml']),
+                ('block_storage_scale', 3)
+            ]
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+            result = self.cmd.take_action(parsed_args)
+        self.assertTrue(result)
+        self.assertTrue(mock_heat_deploy.called)
+        self.assertTrue(mock_oc_endpoint.called)
+        self.assertTrue(mock_create_ocrc.called)
+        self.assertTrue(mock_create_tempest_deployer_input.called)
+
+        # Check that Heat was called with correct parameters:
+        call_args = mock_heat_deploy.call_args[0]
+        self.assertEqual(call_args[3],
+                         '/dev/null/overcloud-without-mergepy.yaml')
+        self.assertIn('/tmp/foo.yaml', call_args[5])
+        self.assertIn('/tmp/environment.yaml', call_args[5])
+        foo_index = call_args[5].index('/tmp/foo.yaml')
+        env_index = call_args[5].index('/tmp/environment.yaml')
+        self.assertGreater(env_index, foo_index)
+
+        mock_create_tempest_deployer_input.assert_called_with()
+
+    @mock.patch('tripleoclient.utils.get_password')
+    @mock.patch('os_cloud_config.keystone.initialize')
+    @mock.patch('os_cloud_config.utils.clients.get_keystone_client')
+    def test_keystone_init(self, mock_gkc, mock_init, mock_gp):
+        mock_ksc = mock.Mock()
+        mock_gkc.return_value = mock_ksc
+        mock_ksc.users.find.return_value = True
+        sips = mock.Mock()
+        sips.get.return_value = None
+        ip = '192.0.2.1'
+        overcloud_deploy.DeployOvercloud(None, None)._keystone_init(ip, ip,
+                                                                    None, sips)
+        self.assertFalse(mock_init.called)
+
+    @mock.patch('tripleoclient.utils.get_password')
+    @mock.patch('os_cloud_config.keystone.setup_endpoints')
+    @mock.patch('os_cloud_config.keystone.initialize')
+    @mock.patch('os_cloud_config.utils.clients.get_keystone_client')
+    def test_keystone_init_occ(self, mock_gkc, mock_init, mock_se, mock_gp):
+        mock_ksc = mock.Mock()
+        mock_gkc.return_value = mock_ksc
+        mock_ksc.users.find.side_effect = kscexc.NotFound()
+        sips = mock.Mock()
+        sips.get.return_value = None
+        ip = '192.0.2.1'
+        args = mock.Mock()
+        overcloud_deploy.DeployOvercloud(None, None)._keystone_init(ip, ip,
+                                                                    args, sips)
+        self.assertTrue(mock_init.called)
